#region Namespaces
using System;
using System.Collections.Generic;
using System.Diagnostics;
using Autodesk.Revit.ApplicationServices;
using Autodesk.Revit.Attributes;
using Autodesk.Revit.DB;
using Autodesk.Revit.DB.Mechanical;
using Autodesk.Revit.UI;
using Autodesk.Revit.UI.Selection;
#endregion

namespace CreateTakeoff
{
    [Transaction(TransactionMode.Manual)]
    public class Command : IExternalCommand
    {
        public Result Execute(ExternalCommandData commandData,
              ref string message,
              ElementSet elements)
        {
            Result result = Result.Failed;

            UIApplication uiapp = commandData.Application;
            Document doc = uiapp.ActiveUIDocument.Document;

            Transaction t = null;

            Autodesk.Revit.Creation.Document creDoc
              = doc.Create;

            try
            {
                // determine duct type to use:

                FilteredElementCollector collector
                  = new FilteredElementCollector(doc);

                collector.OfCategory(
                  BuiltInCategory.OST_DuctCurves);

                collector.OfClass(typeof(
                  ElementType));

                DuctType ductType
                  = collector.FirstElement() as DuctType;

                if (null == ductType)
                {
                    message = ("   ");
                    return Result.Failed;
                }
                else
                {
                    t = new Transaction(doc, " ");

                    t.Start();

                    //  1    
                    // c (0,0,0)  (6,0,0):

                    XYZ start = XYZ.Zero;
                    XYZ end = start + 6 * XYZ.BasisX;
                    XYZ mid = 0.5 * (start + end);

                    Duct duct1 = creDoc.NewDuct(
                      start, end, ductType);

                    //  2   Y
                    //  (1,2,0)  (1,4,0):

                    start = mid + 2 * XYZ.BasisY;
                    end = start + 2 * XYZ.BasisY;

                    Duct duct2 = creDoc.NewDuct(
                      start, end, ductType);

                    //      
                    //   .

                    Connector duct2_start = null;

                    //      
                    //     .
                    //     

                    double dist = double.MaxValue;

                    foreach (Connector c in
                      duct2.ConnectorManager.Connectors)
                    {
                        XYZ p = c.Origin;
                        double d = p.DistanceTo(mid);

                        if (d < dist)
                        {
                            dist = d;
                            duct2_start = c;
                        }
                        break;
                    }

                    //  

                    FamilyInstance takeoff
                      = creDoc.NewTakeoffFitting(
                        duct2_start, duct1);

                    t.Commit();

                    result = Result.Succeeded;
                }
            }
            catch (Exception ex)
            {
                if (null != t)
                {
                    t.RollBack();
                }
                message = ex.Message;
            }
            return result;
        }
    }
}
